/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ti.msp.gui.main;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;

/**
 *
 * @author a0272600
 */
public class KeyIV {
    boolean isKeyValid;
    boolean isIVValid;
    boolean isFileSelected;
    boolean isFileKeyValid;
    boolean isFileIVValid;
    String KeyValue;
    String KeyFormatted;
    String IVValue;
    String IVFormatted;
    List<String> KeyFileLines;
    String Version;
    String Type;

    public static final String typeDEK = "00";
    public static final String typeKEK = "02";
    public static final String typeReserved = "03";

           
        
    public KeyIV(){
        this.isKeyValid = true;
        this.isIVValid = true;
        this.isFileSelected = false;
        this.Type = typeDEK;
        this.Version = "01";
    }
    
    class KeyError extends Exception {
       public KeyError(String message) {
         super(message);
       }
    }
        
    public void FormatKey(String key) throws KeyError{
        this.KeyValue = key;
        int lengthcheck = this.KeyValue.replaceAll("\\s+","").replaceAll(",","").length();

       if(lengthcheck != 32 && lengthcheck != 48 && lengthcheck != 64)
        {
            throw new KeyError("Key length is incorrect. Operation cancelled.");
        }
            
        if(!this.KeyValue.replaceAll("\\s+","").replaceAll(",","").matches("[0-9A-Fa-f]+"))
        {
            throw new KeyError("Key format is incorrect. Operation cancelled.");
        }

        this.KeyFormatted = this.KeyValue.replaceAll("\\s+","").replaceAll(",","");
        for(int i = 0; i<lengthcheck/2-1; i++) 
        {
            this.KeyFormatted = this.KeyFormatted.substring(0,i*3+2) + " " +
                                this.KeyFormatted.substring(i*3+2, this.KeyFormatted.length());
        }
    }
        
    public void FormatIV(String IV) throws KeyError{
        this.IVValue = IV;
        int lengthcheck = this.IVValue.replaceAll("\\s+","").replaceAll(",","").length();

        if(lengthcheck != 26)
        {
            throw new KeyError("Key length is incorrect. Operation cancelled.");
        }

        if(!this.IVValue.replaceAll("\\s+","").replaceAll(",","").matches("[0-9A-Fa-f]+"))
        {
            throw new KeyError("Key format is incorrect. Operation cancelled.");
        }

        this.IVFormatted = this.IVValue.replaceAll("\\s+","").replaceAll(",","");
        for(int i = 0; i<lengthcheck/2-1; i++) 
        {
            this.IVFormatted = this.IVFormatted.substring(0,i*3+2) + " " +
                               this.IVFormatted.substring(i*3+2, this.IVFormatted.length());
        }
    }
        
    public void GetKeyIVFromFile(String file, String type) throws KeyError, IOException 
    {
        try
        {
            // Invalidate file until we validate contents
            this.isFileSelected = false;
            this.KeyFileLines = Files.readAllLines(Paths.get(file));

            // Get Key and IV from file
            this.isFileKeyValid = false;
            FormatKey(this.KeyFileLines.get(0).replaceAll("\\s+","").replaceAll(",","").substring(4));
            this.isFileKeyValid = true;

            this.isFileIVValid = false;
            FormatIV(this.KeyFileLines.get(1));
            this.isFileIVValid = true;

            // Contents are valid
            this.isFileSelected = true;
            this.KeyFileLines = Arrays.asList(this.KeyFormatted, this.IVFormatted);
            this.KeyFileLines.set(0,type + " " + this.Version + " " + this.KeyFileLines.get(0));
            this.Type = type;
        }
        catch (KeyError|IOException err) {
            throw err;
        }
    }
    
    public void GetKeyIVFromFile(String file) throws KeyError, IOException 
    {

        try
        {
            // Invalidate file until we validate contents
            this.isFileSelected = false;
            this.KeyFileLines = Files.readAllLines(Paths.get(file));
            String keyinfo = this.KeyFileLines.get(0).replaceAll("\\s+","").replaceAll(",","").substring(0,4);

            // Get Key and IV from file
            this.isFileKeyValid = false;
            FormatKey(this.KeyFileLines.get(0).replaceAll("\\s+","").replaceAll(",","").substring(4));
            this.isFileKeyValid = true;

            this.isFileIVValid = false;
            FormatIV(this.KeyFileLines.get(1));
            this.isFileIVValid = true;

            // Contents are valid
            this.isFileSelected = true;
            this.KeyFileLines = Arrays.asList(this.KeyFormatted, this.IVFormatted);
            this.KeyFileLines.set(0,keyinfo.substring(0,2) + " " + keyinfo.substring(2,4)  + " " + this.KeyFileLines.get(0));
            this.Type = keyinfo.substring(0,2);
            this.Version = keyinfo.substring(2,4);
        }
        catch (KeyError|IOException err) {
            throw err;
        }
    }
        
    public void GetKeyIVFromText(String key, String IV, String type) throws KeyError
    {

        try
        {
            this.isKeyValid = false;
            FormatKey(key);
            this.isKeyValid = true;
            this.isIVValid = false;
            FormatIV(IV);
            this.isIVValid = true;
            this.KeyFileLines = Arrays.asList(this.KeyFormatted, this.IVFormatted);
            this.KeyFileLines.set(0,type + " " + this.Version + " " + this.KeyFileLines.get(0));
            this.Type = type;
        }
        catch (KeyError err) {
            throw err;
        }
    }
    
    public void GetKeyIVFromText(String key, String type) throws KeyError
    {

        try
        {
            this.isKeyValid = false;
            FormatKey(key);
            this.isKeyValid = true;
            this.isIVValid = false;
            this.KeyFileLines = Arrays.asList(this.KeyFormatted, this.IVFormatted);
            this.KeyFileLines.set(0,type + " " + this.Version + " " + this.KeyFileLines.get(0));
            this.Type = type;
        }
        catch (KeyError err) {
            throw err;
        }
    }
    
    public void SetVersion(String version)
    {
        this.Version = version;
    }
    
    
}
